c ==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+=
      program neutronstar
c ==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+=
c
c     Spherical neutron star in GR 
c     calculated in isotoropic coordinate using KEH iteration.
c     
c ==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+==+=
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_fluid.f'
      include 'common_blocks/fR_NS_grav.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
      include 'common_blocks/CB_fR_param_iomis.f'
c
cref  common / cordp / rvout, radmid
cref  common / calcp / ber, radi
cref  common / physp / pinx, surr, g, pi, emdc
cref  common / fitms / fitadm, fitkom
cref  common / flmas / ahores
cref  common / seqmas/ restmass, gravmass
cref  common / iomis / ffgra, ffden, eps, convini, itmx, iddd, numseq,
cref &                 itype, iwrite, emddiff, rmeps, dr0
c
c
      dimension soupsg(0:nnrg), souapg(0:nnrg), soufpg(0:nnrg)
      dimension potg(0:nnrg), backg(0:nnrg)
c
      character*4 char
      character*1 chrot,chbhs,chgra, chope
c
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
      open(1,file='frgpar.dat',status='old')
      open(24,file='frgaux.dat',status='old')
      open(26,file='frgaux2.dat',status='unknown')
c
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c --- Constants.  
c
      g = 6.6726d-8
      pi = 3.14159265358979d+0
c
c --- Parameters
c ==  frgpar.dat
c
      read(1,5000) nrg, nrf 
      read(1,5002) nrout, dr0, rvout
      read(1,5002) itmx, eps, convini
      read(1,5001) ffgra, ffden
      read(1,5004) itype, iwrite, iddd, chrot, chbhs, chgra, chope
c
      read(1,5000) numseq
      read(1,3) restmass, gravmass
      read(1,3) emdc, pinx
      read(1,3) emddiff, rmeps
c
 5000 format( 5i5)
 5001 format( 2f10.5)
 5002 format( 1i5, 2f10.5)
 5003 format( 1f10.5, 1i4)
 5004 format( 3i5, 3x, 4a1)
 5005 format( 2i5, 1f10.5)
 5011 format(1f10.5,2i5)
 3    format(1p,2e14.6)
 2    format(1f10.5,2i5)
 1    format(2f10.5)
c
      rmepsbak = rmeps
c
      close(1)
c
c +++ Coordinates
c
ccc      nrtot= nrg
      nrin = 2*nrf
      nrtot = nrout + nrin
      radmid = 2.0d0
c
c###  Fluid coordinate
c
      nr = nrf
c
      char = chrot//chbhs//chgra//chope
c
c
      write(6,*)' coordinates etc. '
      call nonvfR
c
c
c ----------------------------------------------------------------------
c --- Initial.
c ----------------------------------------------------------------------
c
      istat = itype
      iseq  = 0
      call subio(istat,iseq,char)
      call subio_fluid(istat,iseq,char)
c
c
c ----------------------------------------------------------------------
c --- Calcuration of a sequence.
c ----------------------------------------------------------------------
c
c --  Initial might be before nonv**.
c
      iseq = 0
c
      do 1400 iseq = 1, numseq
c
cc      if (iseq.eq.1) emdc = emdcc(iseq)
cc      if (rmeps.lt.eps.and.incr.eq.0) emdc = emdcc(iseq)
      surr = 10.0d0**(idnint(dlog10(1.0d-04*emdc)))
c
c
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c     Iteration starts.  
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
      iter = 0
      imascond = 1
      rmeps = rmepsbak
      epsmax = 0.0d0
c
 2444 continue 
c 
      iter = iter + 1 
c
      ermax0 = 0.0d0 
      epsmax = 0.0d0
      fmax0 = 0.0d0
      fmigra = ffgra
      fmiden = ffden
      fcon = convini*dble(iter)
      fffac = dmin1( fmigra, fcon)
      convf = dmin1( fmiden, fcon)
c
c --- Start computation.  
c
      write(6,*) ' #### Iteration NO. = ', iter
c
c ==  Interpolations from fluid.
c
      call al2alps
      call fl2gr(emd,emdg)
c
c --- Compute source terms.
c
      call asymptopia_alps(soupsou,souapou,dsoupsou,dsouapou)
c
      call sourcetermg(soupsg,souapg,soufpg)
c
c --- Call Poisson solver to compute Green's formula.
c
      do 14 ii = 1, 2
ccc      do 14 ii = 1, 3
c      
c
      do 3001 irg = 0, nrtot
      potg(irg) = 0.0d0
      if (ii.eq.1) potg(irg) = psi(irg)
      if (ii.eq.2) potg(irg) = alps(irg)
cc      if (ii.eq.3) potg(irg) = fphi(irg)
      backg(irg) = potg(irg)
 3001 continue
c 
c
      if (ii.eq.1) call poisol(soupsou,dsoupsou,soupsg,potg,0,0,0,ii)
      if (ii.eq.2) call poisol(souapou,dsouapou,souapg,potg,0,0,0,ii)
      if (ii.eq.3) call poisol(soufpou,dsoufpou,soufpg,potg,0,0,0,ii)
c
      if (ii.eq.1)write(6,*)'--- max error in GR coordinate ---'
c
      if (ii.eq.1) write(6,*) '=== psi     === '
      if (ii.eq.2) write(6,*) '=== alpha   === '
      if (ii.eq.3) write(6,*) '=== fphi    === '
      call frgerror(backg,potg,epsmaxg,irger)
      potg0 = potg(irger)
      bakg0 = backg(irger)
      epsmax = dmax1(epsmaxg,epsmax)
      write(6,4905) irger, bakg0, potg0, epsmaxg
c
 4905 format(' GR error ',i5,1p,3e12.4)
c
c --  Update variables.
c
      call potimpro(potg,backg)
c
      do 3011 irg = 0, nrtot
      if (ii.eq.1) psi(irg) = potg(irg)
      if (ii.eq.2) alps(irg) = potg(irg)
cc      if (ii.eq.3) fphi(irg) = potg(irg)
 3011 continue
c 
      if (ii.eq.2) call alps2al
      call paimproco_frg(ber,radi,convf,iter,fmax0,0)
      epsmax = dmax1(fmax0,epsmax)
c
   14 continue
c
c
      call alps2al
      call gr2fl(alph,alphf)
      call gr2fl(psi,psif)
c
c --  Solve First integral of Euler eq.
c
      call fluidco(iter,itmx,convf,cfvep,fmax0,emxemd,char)
      epsmax = dmax1(fmax0,emxemd,epsmax)
c
      call restmass_frg
      call admmass_frg
c
c
      write(6,4902)'  -- epsmax --, error =', epsmax
 4902 format(a22,1p,2e12.4)
c
c --- Untill convergence is made, go to 2444.
c
      if (iter.eq.itmx.or.epsmax.le.eps) go to 2445
      go to 2444
 2445 continue
c
c --- Print out the intermediate state.
      if (iter.eq.itmx.and.epsmax.gt.eps) istat = 2
c --- Print out the converged state.
      if (epsmax.le.eps) istat = 1
c
      call subio(istat,iseq,char)
      call subio_fluid(istat,iseq,char)
      call physq_fR(istat,iseq)
c
      if (iter.eq.itmx.and.epsmax.gt.eps) write(6,*) ' **iter** '
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c --- Calcuration of deformation sequence.
c
      write(6,*)' Sequence No.= ',iseq
c
 1400 continue
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c --- Formats
c
 4444 format(1p,25e12.4)
 4901 format(a22,1p,e12.4,',  value =',e12.4,2x,3i4)
 4000 format(1p, 6e12.4)
 4400 format(1p,17e12.4)
c
c --- end of the program
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine sourcetermg(soupsg,souapg,soufpg)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_grav.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
c
c
      dimension soupsg(0:nnrg), souapg(0:nnrg), soufpg(0:nnrg)
c
      dimension x5(5), f5(5)
c
c --- Compute source terms for volume integrals.
c
      intnum = 5
      intp = 2
      fac13 = 1.0d0/3.0d0
      fac23 = 2.0d0/3.0d0
      fac43 = 4.0d0/3.0d0
      fac512 = 5.0d0/12.0d0
c
c --  GR coordinate
c
      do 100 irg = 0, nrtot
c
      psiw = psi(irg)
      alphw = alph(irg)
      alpsw = alps(irg)
c      fphiw = fphi(irg)
c
c --  For fluid terms
c
      emdgc = emdg(irg)
      psigc =  psi(irg)
      alpgc = alph(irg)
      rhogc = emdgc**pinx
      pregc = rhogc*emdgc
      hhgc  = 1.0d0 + (pinx+1.0d0)*emdgc
c
      utgc  = hhgc/ber
c
      zfac = 1.0d0
      if (emdgc.le.0.0d0) zfac = 0.0d0
      rhoHc = hhgc*rhogc*(alpgc*utgc)**2 - pregc
      rp2s = 3.0d0*hhgc*rhogc*(alpgc*utgc)**2 
     &     - 2.0d0*hhgc*rhogc + 5.0d0*pregc
c
c --  For psi.
c
      soupsg(irg) = 
     &    - radi**2*2.0d0*pi*psigc**5*rhoHc*zfac
c
c --  For alpha*psi.
c
      souapg(irg) = 
     &    + radi**2*2.0d0*pi*alpgc*psigc**5*rp2s*zfac
c
c --  For fphi.
c
      soufpg(irg) = 
     &    + radi**2*2.0d0*pi*alpgc*psigc**5*rp2s*zfac
c
  100 continue
c
c 3400 format(1p,12e14.6) 
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine asymptopia_alps(soupsou,souapou,dsoupsou,dsouapou)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
c
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_cordp.f'
c
c
c --- Compute source for GR boundary terms.
c
c
      soupsou = 1.0d0
      souapou = 1.0d0
      dsoupsou = 0.0d0
      dsouapou = 0.0d0
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine poisol(sousfo,dsousfo,soug,potg,iesy,irsy,imini,ieq)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
c
      dimension soug(0:nnrg),potg(0:nnrg)
      dimension potg1(0:nnrg),
     &          poto1(0:nnrg),poto2(0:nnrg)
c
c --- Poisson solver : call volume and surface integrals.
c
c     sources for volume integral are on half integer points.
c     sources for surface integral are on half integer points.
c
c --- Call integral terms of Green's formula.
c
      call halfsou(soug)
c
      call gravmid_sph(soug,potg1)
      call grav4sfout_sph(dsousfo,poto1,3)
      call grav4sfout_sph( sousfo,poto2,4)
c
      do 110 irg = 0, nrtot
      potg(irg) = potg1(irg)
ctesttest     &          + poto1(irg) + poto2(irg)
     &          + 1.0d0
ctesttest
 110  continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine gravmid_sph(sou,pot)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
c
      common / grfsn / hfsn(0:nnrg,0:nnrg)
      common / weigg / wgdrg(nnrg), w4drg(0:nnrg)
c
      dimension sou(0:nnrg), pot(0:nnrg)
c
c --- Gravitational potential is caluculated. ---
c
c     L[p] = S --> p = 1/4pi Int[ S/r dV]
c              
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c    hfsn(irr,ir) -->  for r'< r
c                      for r < r'
c
      do 1003 ir = 0, nrtot
      pot(ir) = 0.0d0
      do 1003 irr = 1, nrtot
      wei = wgdrg(irr)
      pot(ir) = pot(ir) + sou(irr)*hfsn(irr,ir)*wei
 1003 continue
c
      do 100 ir = 0, nrtot
 100  pot(ir) = - pot(ir)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine grav4sfout_sph(sousf,pot,iaho)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/CB_fR_param_cordp.f'
      common / weigg / wgdrg(nnrg), w4drg(0:nnrg)
c
      dimension  pot(0:nnrg)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
      pi = 3.14159265358979d+0
      pi4inv = 1.0d0/4.0d0/pi
c
      do 100 ir = 0, nrtot
 100  pot(ir) = pi4aho*pot(ir)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine halfsou(soug)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
c
      dimension soug(0:nnrg), sougb(0:nnrg)
      dimension x4(4), f4(4)
c
      do 100 irg = 0, nrtot
      sougb(irg) = soug(irg)
  100 continue
c
      do 101 irg = 1, nrtot
      hhrr = hrg(irg)
      irg0 = min0(max0(irg-2,0),nrtot-3) - 1
      do 102 ii = 1, 4
      x4(ii) = rg(irg0+ii)
  102 continue
c
      do 104 ii = 1, 4
      f4(ii) = sougb(irg0+ii)
  104 continue
      soug(irg) = fn_lagint(x4,f4,hhrr)
  101 continue
c
      soug(0) = 0.0d0
c
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine nonvfR
c
c --- cal.non-variables on grids. (s.t. radius sine cosine etc.)
c
      implicit real*8 (a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_coflu.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
c
      common / weigg / wgdrg(nnrg), w4drg(0:nnrg)
      common / weigh / wgdr(0:nnr), w4dr(0:nnr)
      common / grfsn / hfsn(0:nnrg,0:nnrg)
c
c- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
c
c --- Set up of meshes  ---  ( r(1) = dr ( not the center ) )
c --- The constatnt msym means the symmetry of the configuration.
c
      pi = 3.14159265358979d+0
c
      drdr = 1.0d0/dble(nrf)
      drdrinv = 1.0d0/drdr
      rvdom = rvout - radmid
c
      do 100 ir = 0, nrin
      drg(ir)   = drdr
      drginv(ir) = drdrinv
      drginv2(ir) = 0.5d0*drdrinv
 100  continue
c
c --- For variable mesh.
c
      ratrr = 2.0d0
c
 1400 continue
c
      ratrrb = ratrr
      alge = -(ratrr**(nrout+1)-ratrr-rvdom*drdrinv*(ratrr-1.0d0))
      dalge = dble(nrout+1)*ratrr**nrout - 1.0d0 - rvdom*drdrinv
      ratrr = ratrr + alge/dalge
      error = 2.d0*(ratrr - ratrrb)/(ratrr + ratrrb)
c 
      if (dabs(error).gt.1.d-14) go to 1400 
c
      do 101 ir = nrin+1, nrtot
      drg(ir) = ratrr*drg(ir-1)
      drginv(ir) = 1.0d0/drg(ir)
 101  continue
      do 102 ir = nrin+1, nrtot - 1
      drginv2(ir) = 1.0d0/(drg(ir)+drg(ir+1))
 102  continue
      drginv2(nrtot) = 1.0d0/(drg(nrtot)+drg(nrtot-1))
c
      rg(0) = 0.0d0
      rginv(0) = 0.0d0
      do 110 ir = 1, nrtot
      rg(ir) = rg(ir-1) + drg(ir)
      hrg(ir) = (rg(ir) + rg(ir-1))*0.5d0
      rginv(ir) = 1.0d0/rg(ir)
      hrginv(ir) = 1.0d0/hrg(ir)
  110 continue
c
c
      write(6,12) error, ratrr
      write(6,14) rvout, rg(nrtot)
 12   format(' GR ', 1p,4e23.15)
 14   format('    ', 1p,4e23.15)
      rdet = (rg(nrtot) - rvout)/rvout
      if(rdet.gt.1.d-10)then
      write(6,*) ' bad coordinate GR ', rdet
      stop
      end if
c
c --- weight for GR integration.
c
      san1 = 1.0d0/3.0d0
      san4 = 4.0d0/3.0d0
      san2 = 2.0d0/3.0d0
c
      do 194 ir = 1, nrtot
      wgdrg(ir) = hrg(ir)**2*drg(ir)
      w4drg(ir) = wgdrg(ir)
  194 continue
c
c --- weight for fluid integration.
c
      ir = 0
      wgdr(ir) = 0.5d0*rg(ir)**2*drg(ir+1)
      ir = nrf
      wgdr(ir) = 0.5d0*rg(ir)**2*drg(ir)
      do 195 ir = 1, nrf - 1
      wgdr(ir) = 0.5d0*rg(ir)**2*drg(ir) + 0.5d0*rg(ir)**2*drg(ir+1)
  195 continue
      do 196 ir = 0, nrf
      w4dr(ir) = wgdr(ir)
  196 continue
c
c
c -------------------------------------
c --- Computation of functoin hfsn. ---
c -------------------------------------
c
c    hfsn(irr,ir) --> for r' < r
c                     for r < r'
c
      do 418 ir  = 0, nrtot
      do 418 irr = 0, nrtot
        hfsn(irr,ir) = 0.d0
  418 continue
c
      do 419 ir  = 1, nrtot
      do 4190 irr = 1, nrtot
      if (hrg(irr).lt.rg(ir)) then
        hfsn(irr,ir) = 1.0d0/rg(ir)
      end if
      if (hrg(irr).ge.rg(ir)) then
        hfsn(irr,ir) = 1.0d0/hrg(irr)
      end if
 4190 continue
 419  continue
c
      ir  = 0
      nn  = 0
      do 4200 irr = 1, nrtot
        hfsn(irr,ir) = 1.0d0/hrg(irr)
 4200 continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine frgerror(backg,rnewg,epsmaxg,irger)
c
c --- Compute non-variables for inteporation.
c
      implicit real*8 (a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
      include 'common_blocks/fR_NS_cogra.f' 
c
      dimension rnewg(0:nnrg),backg(0:nnrg)
c
c --- Set improved values for quantities on GR-coordinate and 
c --- convergence check.  
c
      epsmaxg = 0.0d0
      do 401 irg = 0, nrtot
      edet  = rnewg(irg)
      edetb = backg(irg)
      devi  = dabs(rnewg(irg)) + dabs(backg(irg))
c      
      if (irg.le.nrin.and.devi.ge.1.0d-8) then
      error = dabs(2.d0*(edet - edetb))/devi
      if(error .gt. epsmaxg) then
      epsmaxg = error
      irger = irg
      end if
      end if
 401  continue
c
      end 
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine al2alps
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/CB_fR_alps_grav.f'
c
c --  set alps
c
      do 101 irg = 0, nrtot
      alps(irg) = alph(irg)*psi(irg)
  101 continue
c
      end
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine alps2al
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f'
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/CB_fR_alps_grav.f'
c
c --  set alpha
c
      do 103 irg = 0, nrtot
      alph(irg) = alps(irg)/psi(irg)
calph      alps(irg) = alph(irg)*psi(irg)
  103 continue
c
      end
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine physq_fR(istat,iseq,chgra)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_fluid.f'
      include 'common_blocks/fR_NS_grav.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
      include 'common_blocks/CB_fR_param_iomis.f'
c
      common / weigg / wgdrg(nnrg), w4drg(0:nnrg)
      common / weigh / wgdr(0:nnr), w4dr(0:nnr)
c
c
      character*1 chrot,chbhs,chgra, chope
c
      if (iseq.eq.1) then
      open(10,file='frgphy.dat',status='old')
      open(11,file='frgphydp.dat',status='old')
      open(31,file='frgphyplot.dat',status='old')
      open(34,file='frgasym.dat',status='old')
      end if
      if (istat.eq.2) write(10,*) ' == Solution does not converge. == '
c
ctesttest
      ahokom = 0.0d0
      virial = 0.0d0
      admmas = 0.0d0
      rkomas = 0.0d0
      fitvir = 0.0d0
      fitadm = 0.0d0
      fitkom = 0.0d0
ctesttest
c
      write(10,4001) nrf, nrg, pinx, rvout
      write(10,4000) radi, ber, emdc
      write(10,4000) fitvir, fitadm, fitkom
      write(10,4000) ahores, restmass
      write(10,4000) ahoadm, gravmass
c
      write(6,4001) nrf, nrg, pinx, rvout
      write(6,4000) radi, ber, emdc
      write(6,4000) fitvir, fitadm, fitkom
      write(6,4000) ahores, restmass
      write(6,4000) ahoadm, gravmass
c
 4000 format(1p,6e14.6)
 4001 format(2i7, 1p,6e14.6)
 4002 format(3i7,7x,1p,e14.6)
c
c
      write(11,4001) nrf, nrg, pinx
      write(11,4004) radi, ber
      write(11,4004) emdc
      write(11,4004) ahores, ahoadm, ahokom 
      write(11,4004) virial, admmas, rkomas
      write(11,4004) fitvir, fitadm, fitkom
      write(11,4004) restmass, gravmass
 4005 format(3i6, 1p,6e12.4)
 4004 format(1p,3e23.15)
c
      rhogc = emdc**pinx
      pregc = emdc*rhogc
      write(31,4045) radi, emdc, rhogc, pregc, 
     &               ahores, ahoadm, ahokom, 
     &               virial, admmas, rkomas, 
     &               fitvir, fitadm, fitkom, restmass, gravmass
c
 4045 format(1p,40e16.8)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine subio(istat,iseq,char)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_grav.f'
c
      include 'common_blocks/CB_fR_param_cordp.f'
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
      include 'common_blocks/CB_fR_param_iomis.f'
c
c
      dimension rgtmp(0:nnrg)
c
      character*4 char
      character*3 moji
c
c ------------------------------------------------------------
c     istat = 0 and iseq 0 -> open ini file and close it. 
c     istat = 1            -> open las file and close it.
c     istat = 2            -> open nxt file and close it.
c ------------------------------------------------------------
c
      moji = 'nxt'
      if (istat.eq.0.and.iseq.eq.0) moji = 'ini'
      if (istat.eq.1)               moji = 'las'
      if (istat.eq.2)               moji = 'nxt'
      open(3,file='frggra.'//moji,status='old')
c
c -------------
c --- Test. ---
c -------------
c
      if (istat.eq.-1) then
c
      do 20 irg = 0, nrtot
c
      psi(irg) = 1.0d0
      alph(irg) = 1.0d0
      if (irg.eq.0)
     &    alph(irg) = 0.9d0 - emdc
      if (irg.ge.1.and.irg.le.nrf) 
     &    alph(irg) = 0.9d0 - emdc*dsin(pi*rg(irg))/(pi*rg(irg))
c
   20 continue
c
      end if
c
c -------------
c --- Read. ---
c -------------
c
      if (istat.eq.0) then
c
c --- Metric potentials.  
c
      read(3,5000) nrgtmp
      do 200 irg = 0, nrgtmp
      read(3,4000) rgtmp(irg), psi(irg), alph(irg)
 200  continue
      read(3,4000) ber, radi
c
      if (nrgtmp.ne.nrtot) then 
      call interpg_ini(nrgtmp,rgtmp,psi)
      call interpg_ini(nrgtmp,rgtmp,alph)
      end if
c
      do 1200 irg = 0, nrtot
      write(24,8080) irg,rg(irg), psi(irg), alph(irg)
      write(24,8080) irg,rg(irg), psi(irg), alph(irg)
 1200  continue
c        stop
 8080   format(1i5,1p,6e14.6)
c
      close(3)
c
      end if
c
c --------------
c --- Write. ---
c --------------
c
      if (istat.eq.1.or.istat.eq.2) then
c
c --- Metric potentials.  
c
      write(3,5000) nrtot
      do 2000 irg = 0, nrtot
      write(3,4000) rg(irg), psi(irg), alph(irg), emdg(irg)
 2000 continue
      write(3,4000) ber, radi
c
      write(3,5000) nrg, nrf 
      write(3,5002) nrout, dr0, rvout
      write(3,5002) itmx, eps, convini
      write(3,5001) ffgra,ffden
      write(3,5004) itype, iwrite, iddd, char
c
      write(3,5000) numseq
      write(3,3) restmass, gravmass
      write(3,3) emdc, pinx
      write(3,3) emddiff, rmeps
c
      close(3)
c
c --  For plotter
c
      open(3,file='frggraplot.dat',status='unknown')
      do 2010 irg = 0, nrtot
      write(3,4000) rg(irg), psi(irg), alph(irg), emdg(irg)
 2010 continue
      close(3)
c
      end if
c
 4000 format(1p,6e20.12)
 4999 format(' ')
c
 5000 format( 5i5)
 5001 format( 1p,2e10.3)
 5002 format( 1i5, 1p,2e10.3)
 5003 format( 1p,1e10.3, 1i4)
 5004 format( 3i5, 3x, a4)
 5005 format( 2i5, 1p,1e10.3)
 5010 format(1p,3e10.3)
 5011 format(1p,1e10.3,2i5)
c
 3    format(1p,2e14.6)
 2    format(1p,1e10.3,2i5)
 1    format(1p,2e10.3)
c
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine interpg_ini(nrgtmp,rgtmp,grv)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
c
      dimension rgtmp(0:nnrg), grv(0:nnrg), grvbk(0:nnrg)
      dimension x4(4), f4(4)
c
      do 10 irg = 0, nrgtmp 
      grvbk(irg) = grv(irg)
      grv(irg) = 0.0d0
 10   continue
      grv(0) = grvbk(0)
c
      do 100 irg = 1, nrtot
      irg0 = 0
      do 101 irr = 1, nrgtmp
      if (rg(irg).ge.rgtmp(irr-1).and.rg(irg).lt.rgtmp(irr)) then
      fac = (rg(irg) - rgtmp(irr-1))/(rgtmp(irr) - rgtmp(irr-1))
      grv(irg) = fac*grvbk(irr) + (1.0d0 - fac)*grvbk(irr-1)
      go to 100
      end if
 101  continue
      if (rg(irg).ge.rgtmp(nrgtmp)) grv(irg) = grvbk(nrgtmp)
 100  continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine minv(aa,bb,nn,nnz)
c
c --- linear equations solver.
c
      implicit real*8(a-h,o-z)
c
      dimension aa(nnz,nnz),bb(nnz)
c
c     solution of sm*aa=ss
      nnp1 = nn + 1
      do 100 ii = 1, nn
      aa(ii,nnp1) = bb(ii)
  100 continue
c
      nnm1 = nn - 1
      do 200 k = 1, nnm1
      kp1 = k + 1
c
      fmax = 1.e-10
cccp      fmax = 0.0
      ifmax = 1
c
      do 300 iia = k, nn
      aab = abs(aa(iia,k))
      if(aab.gt.fmax) then
         fmax = aab
         ifmax = iia
      end if
  300 continue
      if(ifmax.ne.k) then
        sig = 1.0d0
        if(aa(k,k)*aa(ifmax,k).lt.0.) sig = -1.0d0
        do 310 ii = 1, nnp1
        aa(k,ii) = aa(k,ii) + sig*aa(ifmax,ii)
  310   continue
      end if
c
      do 210 j = kp1, nnp1
      aa(k,j) = aa(k,j)/aa(k,k)
  210 continue
      do 220 i = kp1, nn
      do 220 j = kp1, nnp1
      aa(i,j) = aa(i,j) - aa(i,k)*aa(k,j)
  220 continue
c
  200 continue
c
      aa(nn,nnp1) = aa(nn,nnp1)/aa(nn,nn)
      do 250 l = 1, nnm1
      k   = nn - l
      kp1 = k + 1
      do 260 j = kp1, nn
      aa(k,nnp1) = aa(k,nnp1) - aa(k,j)*aa(j,nnp1)
  260 continue
  250 continue
c
      do 500 ii = 1, nn
      bb (ii) = aa(ii,nnp1)
  500 continue
c
      return
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      function d2fdx2_3rd(xg,fnc,rv)
      implicit real*8 (a-h,o-z), integer (i-n)
      dimension xg(5),fnc(5)
c
      ir0 = 1
      ir1 = 2
      ir2 = 3
      ir3 = 4
      ir4 = 5
      dr01 = xg(ir0) - xg(ir1)
      dr02 = xg(ir0) - xg(ir2)
      dr03 = xg(ir0) - xg(ir3)
      dr04 = xg(ir0) - xg(ir4)
      dr12 = xg(ir1) - xg(ir2)
      dr13 = xg(ir1) - xg(ir3)
      dr14 = xg(ir1) - xg(ir4)
      dr23 = xg(ir2) - xg(ir3)
      dr24 = xg(ir2) - xg(ir4)
      dr34 = xg(ir3) - xg(ir4)
      dr10 = - dr01
      dr20 = - dr02
      dr21 = - dr12
      dr30 = - dr03
      dr31 = - dr13
      dr32 = - dr23
      dr40 = - dr04
      dr41 = - dr14
      dr42 = - dr24
      dr43 = - dr34
      rrv0 = rv - xg(ir0)
      rrv1 = rv - xg(ir1)
      rrv2 = rv - xg(ir2)
      rrv3 = rv - xg(ir3)
      rrv4 = rv - xg(ir4)
      wer0 = 2.0d0*(rrv3*rrv4 + rrv2*rrv4 + rrv2*rrv3
     &            + rrv1*rrv4 + rrv1*rrv3 + rrv1*rrv2)
     &            /(dr01*dr02*dr03*dr04) 
      wer1 = 2.0d0*(rrv3*rrv4 + rrv2*rrv4 + rrv2*rrv3
     &            + rrv0*rrv4 + rrv0*rrv3 + rrv0*rrv2)
     &            /(dr10*dr12*dr13*dr14) 
      wer2 = 2.0d0*(rrv3*rrv4 + rrv1*rrv4 + rrv1*rrv3
     &            + rrv0*rrv4 + rrv0*rrv3 + rrv0*rrv1)
     &            /(dr20*dr21*dr23*dr24) 
      wer3 = 2.0d0*(rrv2*rrv4 + rrv1*rrv4 + rrv1*rrv2
     &            + rrv0*rrv4 + rrv0*rrv2 + rrv0*rrv1)
     &            /(dr30*dr31*dr32*dr34) 
      wer4 = 2.0d0*(rrv2*rrv3 + rrv1*rrv3 + rrv1*rrv2
     &            + rrv0*rrv3 + rrv0*rrv2 + rrv0*rrv1)
     &            /(dr40*dr41*dr42*dr43) 
c
      d2fdx2_3rd = wer0*fnc(ir0) + wer1*fnc(ir1) +
     &             wer2*fnc(ir2) + wer3*fnc(ir3) +
     &             wer4*fnc(ir4)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      function fn_linint(x1,x2,y1,y2,v)
      implicit real*8 (a-h,o-z), integer (i-n)
c
      dx12 = x1 - x2
      dx21 = - dx12
      xv1 = v - x1
      xv2 = v - x2
      wex1 = xv2/dx12
      wex2 = xv1/dx21
c
      fn_linint = wex1*y1 + wex2*y2
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      function fn_lagint(x,y,v)
      implicit real*8 (a-h,o-z), integer (i-n)
      dimension x(4),y(4)
c
      dx12 = x(1) - x(2)
      dx13 = x(1) - x(3)
      dx14 = x(1) - x(4)
      dx23 = x(2) - x(3)
      dx24 = x(2) - x(4)
      dx34 = x(3) - x(4)
      dx21 = - dx12
      dx31 = - dx13
      dx32 = - dx23
      dx41 = - dx14
      dx42 = - dx24
      dx43 = - dx34
      xv1 = v - x(1)
      xv2 = v - x(2)
      xv3 = v - x(3)
      xv4 = v - x(4)
      wex1 = xv2*xv3*xv4/(dx12*dx13*dx14)
      wex2 = xv1*xv3*xv4/(dx21*dx23*dx24)
      wex3 = xv1*xv2*xv4/(dx31*dx32*dx34)
      wex4 = xv1*xv2*xv3/(dx41*dx42*dx43)
c
      fn_lagint = wex1*y(1) + wex2*y(2) + wex3*y(3) + wex4*y(4)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      function fn_lagint5(x,y,v)
      implicit real*8 (a-h,o-z), integer (i-n)
      dimension x(5),y(5)
c
      dx12 = x(1) - x(2)
      dx13 = x(1) - x(3)
      dx14 = x(1) - x(4)
      dx15 = x(1) - x(5)
      dx23 = x(2) - x(3)
      dx24 = x(2) - x(4)
      dx25 = x(2) - x(5)
      dx34 = x(3) - x(4)
      dx35 = x(3) - x(5)
      dx45 = x(4) - x(5)
      dx21 = - dx12
      dx31 = - dx13
      dx32 = - dx23
      dx41 = - dx14
      dx42 = - dx24
      dx43 = - dx34
      dx51 = - dx15
      dx52 = - dx25
      dx53 = - dx35
      dx54 = - dx45
      xv1 = v - x(1)
      xv2 = v - x(2)
      xv3 = v - x(3)
      xv4 = v - x(4)
      xv5 = v - x(5)
      wex1 = xv2*xv3*xv4*xv5/(dx12*dx13*dx14*dx15)
      wex2 = xv1*xv3*xv4*xv5/(dx21*dx23*dx24*dx25)
      wex3 = xv1*xv2*xv4*xv5/(dx31*dx32*dx34*dx35)
      wex4 = xv1*xv2*xv3*xv5/(dx41*dx42*dx43*dx45)
      wex5 = xv1*xv2*xv3*xv4/(dx51*dx52*dx53*dx54)
c
      fn_lagint5 = wex1*y(1)+wex2*y(2)+wex3*y(3)+wex4*y(4)+wex5*y(5)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      function dfncdx(xg,fnc,rb)
      implicit real*8 (a-h,o-z), integer (i-n)
      dimension xg(5),fnc(5)
c
      ir0 = 1
      ir1 = 2
      ir2 = 3
      ir3 = 4
      ir4 = 5
      dr01 = xg(ir0) - xg(ir1)
      dr02 = xg(ir0) - xg(ir2)
      dr03 = xg(ir0) - xg(ir3)
      dr04 = xg(ir0) - xg(ir4)
      dr12 = xg(ir1) - xg(ir2)
      dr13 = xg(ir1) - xg(ir3)
      dr14 = xg(ir1) - xg(ir4)
      dr23 = xg(ir2) - xg(ir3)
      dr24 = xg(ir2) - xg(ir4)
      dr34 = xg(ir3) - xg(ir4)
      dr10 = - dr01
      dr20 = - dr02
      dr21 = - dr12
      dr30 = - dr03
      dr31 = - dr13
      dr32 = - dr23
      dr40 = - dr04
      dr41 = - dr14
      dr42 = - dr24
      dr43 = - dr34
      rrb0 = rb - xg(ir0)
      rrb1 = rb - xg(ir1)
      rrb2 = rb - xg(ir2)
      rrb3 = rb - xg(ir3)
      rrb4 = rb - xg(ir4)
      wer0 = (rrb2*rrb3*rrb4 + rrb1*rrb3*rrb4 +
     &        rrb1*rrb2*rrb4 + rrb1*rrb2*rrb3)/(dr01*dr02*dr03*dr04)
      wer1 = (rrb2*rrb3*rrb4 + rrb0*rrb3*rrb4 +
     &        rrb0*rrb2*rrb4 + rrb0*rrb2*rrb3)/(dr10*dr12*dr13*dr14)
      wer2 = (rrb1*rrb3*rrb4 + rrb0*rrb3*rrb4 +
     &        rrb0*rrb1*rrb4 + rrb0*rrb1*rrb3)/(dr20*dr21*dr23*dr24)
      wer3 = (rrb1*rrb2*rrb4 + rrb0*rrb2*rrb4 +
     &        rrb0*rrb1*rrb4 + rrb0*rrb1*rrb2)/(dr30*dr31*dr32*dr34)
      wer4 = (rrb1*rrb2*rrb3 + rrb0*rrb2*rrb3 +
     &        rrb0*rrb1*rrb3 + rrb0*rrb1*rrb2)/(dr40*dr41*dr42*dr43)
      dfncdx = wer0*fnc(ir0) + wer1*fnc(ir1) +
     &         wer2*fnc(ir2) + wer3*fnc(ir3) +
     &         wer4*fnc(ir4)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine potimpro(potg,backg)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/CB_fR_param_iomis.f'
c
      dimension potg(0:nnrg), backg(0:nnrg)
c
      do 3005 irg = 0, nrtot
      potg(irg) = (1.0d0 - fffac)*backg(irg)
     &                   + fffac * potg(irg)
 3005 continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine adjustasymptopia_bh(isaj)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/CB_fR_alps_grav.f'
c
      include 'common_blocks/CB_fR_param_calcp.f'
c
      dimension psidat(0:nnrg), alphdat(0:nnrg)
c
c --  Fit psi    = 1 + MADM/2r, and alpha   = 1 - MK/r
c         psidat = 0.5/(psi - 1),   alphdat = 1/(1 - alpha)
c
c     isaj = 0 -> adjust psi and alpha in the asymptotics.
c     isaj = 1 -> no adjust
c
      pi = 3.14159265358979d+0
c
      ave = dble(ntg+1)*dble(npg+1)
      irg10 = nrtot
      do 4 irg = nrtot, 0, -1
      if (ome.ne.0.0d0) then 
      if (rg(irg).ge.10.0d0*pi/ome) irg10 = irg
      end if
    4 continue
      irgini = min0(nrtot-15,irg10)
      nrgend = nrtot - 5
      npoi = nrgend - irgini + 1
c
      do 10 irg = irgini, nrgend
      psiv  = 0.0d0
      alphv = 0.0d0
      psiv  = psiv  + psi(irg)
      alphv = alphv + alph(irg)
      psidat(irg)  = 0.5d0/(psiv/ave - 1.0d0)
      alphdat(irg) = 1.0d0/(1.0d0 - alphv/ave)
   10 continue
c
      s1 = 0.0d0
      sp2 = 0.0d0
      sa2 = 0.0d0
      do 30 irg = irgini, nrgend
      s1  = s1  + rg(irg)**2
      sp2 = sp2 + psidat(irg)*rg(irg)
      sa2 = sa2 + alphdat(irg)*rg(irg)
   30 continue
c
      coadm = s1/sp2
      cokom = s1/sa2
      fitadm = 0.5d0*coadm
      fitkom = 0.5d0*cokom
c
      fitvir = dabs((fitadm-fitkom)/fitadm)
c
      if (isaj.eq.0) then 
      do 40 irg = irgini, nrtot
      psi(irg)  = 1.0d0 + coadm/(2.0d0*rg(irg))
      alph(irg) = 1.0d0 - cokom/rg(irg)
   40 continue
      end if
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine subio_fluid(istat,iseq,char)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_fluid.f'
      include 'common_blocks/CB_fR_param_cordp.f'
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
      include 'common_blocks/CB_fR_param_iomis.f'
c
      dimension rftmp(0:nnr)
c
      character*4 char
      character*3 moji
c
c ------------------------------------------------------------
c     istat = 0 and iseq 0 -> open ini file and close it 
c                             and open las file
c     istat = 2            -> close las file and open nxt file
c ------------------------------------------------------------
c
      moji = 'nxt'
      if (istat.eq.0.and.iseq.eq.0) moji = 'ini'
      if (istat.eq.1)               moji = 'las'
      if (istat.eq.2)               moji = 'nxt'
      open(2,file='frgflu.'//moji,status='old')
c
c -------------
c --- Test. ---
c -------------
c
      if (istat.eq.-1) then
c
      emd(0) = emdc
      do 20 irf = 1, nrf
      emd(irf) = emdc*dsin(pi*rg(irf))/(pi*rg(irf))
   20 continue
      ber = 0.8d0
      radi = 0.9d0
c
      return
      end if
c
c -------------
c --- Read. ---
c -------------
c
      if (istat.eq.0) then
c
c --- Fluid variables
c
      read(2,5000) nrftmp
      do 100 ir = 0, nrftmp
      read(2,4000) rftmp(ir), emd(ir)
 100  continue
      read(2,4000) ber, radi
c
      if (nrftmp.ne.nrf) call interpf_ini(nrftmp,rftmp,emd)
c
      end if
c
c --------------
c --- Write. ---
c --------------
c
      if (istat.ne.0) then
c
c --- Fluid variables
c
      write(2,5000) nrf
      do 400 ir = 0, nrf
      write(2,4000) rg(ir), emd(ir)
 400  continue
      write(2,4000) ber, radi
c
      write(2,5000) nrg, nrf 
      write(2,5002) nrout, dr0, rvout
      write(2,5002) itmx, eps, convini
      write(2,5001) ffgra,ffden
      write(2,5004) itype, iwrite, iddd, char
c
      write(2,5000) numseq
      write(2,3) restmass, gravmass
      write(2,3) emdc, pinx
      write(2,3) emddiff, rmeps
c
      end if
c
      close(2)
c
 4000 format(1p,6e20.12)
 4999 format(' ')
c
 5000 format( 5i5)
 5001 format( 1p,2e10.3)
 5002 format( 1i5, 1p,2e10.3)
 5003 format( 1p,1e10.3, 1i4)
 5004 format( 3i5, 3x, a4)
 5005 format( 2i5, 1p,1e10.3)
 5010 format(1p,3e10.3)
 5011 format(1p,1e10.3,2i5)
c
 3    format(1p,2e14.6)
 2    format(1p,1e10.3,2i5)
 1    format(1p,2e10.3)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine interpf_ini(nrftmp,rftmp,flv)
c
      implicit real*8(a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
c
      dimension rftmp(0:nnr), flv(0:nnr), flvbk(0:nnr)
      dimension x4(4), f4(4)
c
      do 10 irf = 0, nrftmp 
      flvbk(irf) = flv(irf)
      flv(irf) = 0.0d0
 10   continue
      flv(0) = flvbk(0)
c
      do 100 irf = 1, nrf
      irf0 = 0
      do 101 irr = 1, nrftmp
      if (rg(irf).ge.rftmp(irr-1).and.rg(irf).lt.rftmp(irr)) then
      fac = (rg(irf) - rftmp(irr-1))/(rftmp(irr) - rftmp(irr-1))
      flv(irf) = fac*flvbk(irr) + (1.0d0 - fac)*flvbk(irr-1)
      go to 100
      end if
 101  continue
      if (rg(irf).ge.rftmp(nrftmp)) flv(irf) = flvbk(nrftmp)
 100  continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine flimpro(back,rnew,fffac,epsmax,irerr,iterr,iperr,isw)
c
c --- Compute non-variables for inteporation.
c
      implicit real*8 (a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
c
      dimension rnew(0:nnr),
     &          back(0:nnr)
c
c --- Set improved values for quantities on fluid-coordinate and 
c --- convergence check.  
c
      epsmax = 0.0d0
      do 401 ir = 0, nr - isw
      rnew(ir) = 
     &   fffac*rnew(ir) + (1.d0-fffac)*back(ir)
      edet  = rnew(ir)
      edetb = back(ir)
      devi  = dabs(rnew(ir)) + dabs(back(ir))
      if (dabs(edet+edetb).ge.1.0d-8) then
      error = dabs(2.d0*(edet - edetb))/devi
      if(error .gt. epsmax) then
      epsmax = error
      irerr = ir
      end if
      end if
 401  continue
c
      end 
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine paimproco_frg(ber,radi,convf,iter,fmax0,isw)
c
c --- Improve parameters
c
      implicit real*8 (a-h,o-z), integer (i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
      include 'common_blocks/fR_NS_fluid.f'
      include 'common_blocks/fR_NS_grav.f'
c
      include 'common_blocks/CB_fR_param_physp.f'
c
c
c --- for interporation.
c
      nrgsf = nrf
c
      emdmid = emdc
      hhmid  = 1.d0+(pinx+1.d0)*emdmid
      hhsf   = 1.d0+(pinx+1.d0)*surr
      dloghmid = dlog(hhmid)
      dloghsf  = dlog(hhsf)
c
      f0 = alph(0)
      f1 = alph(nrf)
      dlogamid = dlog(f0**(1.d0/radi**2))
      dlogasf = dlog(f1**(1.d0/radi**2))
c
      berb = ber
      radib = radi
c
c
      radi = dsqrt(dabs(-(dloghmid - dloghsf)/(dlogamid - dlogasf)))
      ber  = dexp(0.5d0*(radi**2*(dlogamid + dlogasf)
     &                         + (dloghmid + dloghsf)))
c
c
      if (radi.le.0.d0) write(6,*) ' ### radi minus ###'
      if (radi.le.0.d0) radi = - radi
      if (ber.le.0.d0) write(6,*) ' ### ber minus ###'
      if (ber.le.0.d0) ber = - ber
      ber = convf*ber + (1.d0-convf)*berb
      radi = convf*radi + (1.d0-convf)*radib
c
      if (isw.eq.1) then
      write(6,4900)'  -- ber   --, error =', 
     &        2.0d0*abs((ber - berb)/(ber + berb)), ber
      write(6,4900)'  -- radi  --, error =', 
     &        2.0d0*dabs((radi - radib)/(radi + radib)), radi
      end if
      fmax0 = dmax1(2.0d0*dabs((ber - berb)/(ber + berb)),
     &              2.0d0*dabs((radi - radib)/(radi + radib)),fmax0)
c
c --  Improving alpha and psi.  
c
      do 470 irg = 0, nrtot
      alph(irg) = alph(irg)**((radi/radib)**2)
      psi(irg) = psi(irg)**((radi/radib)**2)
 470  continue
c
 4900 format(a22,1p,e12.4,',  value =',e12.4)
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine fl2gr(flv,grv)
c
      implicit real*8(a-h,o-z), integer(i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
      dimension flv(0:nnr), grv(0:nnrg)
c
      do 100 irg = 0, nrtot
      grv(irg) = 0.0d0
 100  continue
      do 101 irf = 0, nrf
      grv(irf) = flv(irf)
 101  continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine gr2fl(grv,flv)
c
      implicit real*8(a-h,o-z), integer(i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_cogra.f'
      dimension flv(0:nnr), grv(0:nnrg)
c
      do 100 irf = 0, nrf
      flv(irf) = grv(irf)
 100  continue
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine fluidco(iter,itmx,convf,cfvep,fmax0,emxemd,char)
c
      implicit real*8(a-h,o-z), integer(i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_fluid.f'
c
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
c
      common / weigh / wgdr(0:nnr), w4dr(0:nnr)
c
c
      dimension backf(0:nnr)
      dimension hut(0:nnr), aloh(0:nnr),
     &        hutp6(0:nnr),psif4(0:nnr)
c
      character*3 char
c
c ----------------------------------------------------------------------
c
      dr = 1.0d0/dble(nr)
      drf = dr
      drinv=1.0d0/dr
      drfinv=drinv
      pi = 3.14159265358979d+0
c
c ----------------------------------------------------------------------
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
c
c     Equations start. 
c
c ======================================================================
c
c     First integral of Euler equation.
c
c ======================================================================
c
c --  First integral of Euler equation.
c
      do 100 ir = 0, nr
c
      backf(ir) = emd(ir)
c
      alphw = alphf(ir)
c
      emd(ir) = 1.0d0/(pinx+1.0d0)
     &              * (ber/alphw - 1.0d0)
c
  100 continue
c
      emd(0) = emdc
c
      emdmx = 0.0d0
      do 44 ir = 0, nr
      emdmx = dmax1(emdmx,emd(ir))
      if (emdmx.eq.emd(ir)) nrfdmx = ir
 44   continue
c
      emd(0) = emdc
c
c -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
c
c     ### Iteration. ###
c     Set improved values for emden function and surface.
c
c -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
c
c --- For parameters. 
c
      emd(nr) = surr
      do 4000 ir = 0, nr-1
      if (emd(ir).le.0.0d0) emd(ir) = surr
 4000 continue
c
      call flimpro(backf,emd,convf,emxemd,irerr,iterr,iperr,1)
      write(6,4901) '  == emd   ==, error =', emxemd,
     &              emd(irerr), irerr
c
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c --  Improving parameters and alpha and psi.  
c
      call paimproco_frg(ber,radi,convf,iter,fmax0,1)
c
c -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
cmark
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
c
 4901 format(a22,1p,e12.4,',  value =',e12.4,2x,3i4)
c
c - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
      end
c
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine restmass_frg
c
      implicit real*8(a-h,o-z), integer(i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_fluid.f'
c
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
c
      common / weigh / wgdr(0:nnr), w4dr(0:nnr)
c
c
      ahores = 0.0d0
      do 500 ir = 0, nr
c
      emd(nr) = 1.0d-14
      if (emd(ir).le.0.0d0) emd(ir) = 1.0d-14
      emdfc = emd(ir)
      rhofc = emd(ir)**pinx
      alpfc = alphf(ir)
      psifc = psif(ir)
      hhfc  = 1.0d0 + (pinx+1.0d0)*emdfc
      utfc  = hhfc/ber
c
      weiflu = wgdr(ir)
      ahores = ahores + rhofc*alpfc*utfc*psifc**6*weiflu
c
 500  continue
c
      ahores = radi**3*ahores*4.0d0*pi
c
      end
c ______________________________________________________________________
c ______________________________________________________________________
c
      subroutine admmass_frg
c
      implicit real*8(a-h,o-z), integer(i-n)
c
      include 'fRparm.f' 
c
      include 'common_blocks/fR_NS_coflu.f'
      include 'common_blocks/fR_NS_fluid.f'
c
      include 'common_blocks/CB_fR_param_calcp.f'
      include 'common_blocks/CB_fR_param_physp.f'
      include 'common_blocks/CB_fR_param_flphy.f'
c
      common / weigh / wgdr(0:nnr), w4dr(0:nnr)
c
c
      ahoadm = 0.0d0
      do 500 ir = 0, nr
c
      emd(nr) = 1.0d-14
      if (emd(ir).le.0.0d0) emd(ir) = 1.0d-14
      emdfc = emd(ir)
      rhofc = emd(ir)**pinx
      prefc = rhofc*emdfc
      alpfc = alphf(ir)
      psifc = psif(ir)
      hhfc  = 1.0d0 + (pinx+1.0d0)*emdfc
      utfc  = hhfc/ber
c
      rhoHc = hhfc*rhofc*(alpfc*utfc)**2 - prefc
      sou = rhoHc*psifc**5
c
      weiflu = wgdr(ir)
      ahoadm = ahoadm + sou*weiflu
c
 500  continue
c
      ahoadm = radi**3*ahoadm*4.0d0*pi
c
      end
